/* Copyright 2013 Chili Code Solutions, Inc.
 * http://www.chilicode.ca
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

(function($){
	$.fn.chiliSwitch = function(opts) {
		var checkboxes = this.filter('input[type=checkbox]'),
			defaults = {
				on: 'On',
				off: 'Off'
			},
			settings = $.extend({}, defaults, opts);

		checkboxes.hide();
		

		checkboxes.each(function(){
			var
				// The important bits
				$this = $(this),
				$elm = $('<div class="chSw"><div class="chSwSl"><div class="chSwSlLe">'+settings.on+'</div><div class="chSwSlRi">'+settings.off+'</div><div class="chSwSlH"></div></div></div>'),

				// Slider contents
				$slider = $('.chSwSl',$elm),
				$txtOn = $('.chSwSlLe',$elm),
				$txtOff = $('.chSwSlRi',$elm),
				$handle = $('.chSwSlH',$elm),
				$txts = $('.chSwSlLe, .chSwSlRi', $elm),

				// misc closures
				offset = -1,
				delta = 0,
				max = 0,
				ignoreUpdate = false;
			;

			var clearSelection = function() {
				if (window.getSelection) {
				  if (window.getSelection().empty) {  // Chrome
				    window.getSelection().empty();
				  } else if (window.getSelection().removeAllRanges) {  // Firefox
				    window.getSelection().removeAllRanges();
				  }
				} else if (document.selection) {  // IE?
				  document.selection.empty();
				}
			}

			this.turnOn = function(evt) {
				$slider.stop(true).animate({ left: 0 },'fast');
				ignoreUpdate = true;
				$this.prop("checked", false);
				evt.stopPropagation();
			}

			this.turnOff = function(evt) {
				$slider.stop(true).animate({ left: max },'fast');
				ignoreUpdate = true;
				$this.prop("checked", true);
				evt.stopPropagation();
			}

			this.toggle = function(){
				// If we dragged it, delta will be high
				if(Math.abs(delta) < 4) {
					// Delta is low-- probaby a straight-up click (with 3px margin of error)
					var pos = $slider.position();
					ignoreUpdate = true;
					if(pos.left < max / 2) {
						$slider.stop(true).animate({ left: max },'fast');
						$this.prop("checked", true);
					} else if(pos.left >= max / 2) {
						$slider.stop(true).animate({ left: 0 },'fast');
						$this.prop("checked", false);
					}
				}
				delta = 0;
			}

			this.beginDrag = function(x){
				offset = x - ($slider.offset()).left;
				delta = ($slider.position()).left;
			}

			this.drag = function(x){
				// If we started a drag, offset will be > -1
				if(offset > -1) {
					// stop selecting the damn text!! (barely works... don't know why)
					clearSelection();
					var left = Math.max(0, Math.min((x - offset - ($elm.offset()).left), max));
					$slider.css({ 'left': left+'px' });
				}
			}

			this.endDrag = function(){
				if(offset > -1) {
					// We've stopped dragging... reset the offset value
					offset = -1;
					// Complete an incomplete drag
					var pos = $slider.position();
					delta -= pos.left;
					if(pos.left > 0 && pos.left < max / 2) {
						$slider.stop(true).animate({ left: 0 },'fast');
					} else if(pos.left < max && pos.left >= max / 2) {
						$slider.stop(true).animate({ left: max },'fast');
					}
					// Toggle the select
					ignoreUpdate = true;
					$this.prop("checked", (pos.left > max / 2));
				}
			}

			this.update = function(){
				if($this.prop('checked')) {
					$slider.stop(true).animate({ left: max },'fast');
				} else {
					$slider.stop(true).animate({ left: 0 },'fast');
				}
			}
			
			// Add Default style to the page. Append it before any other stylesheet, etc.
			if(!$('.chBaseStyle').length) {
				$('<style type="text/css" class="chBaseStyle">.chSw{display:inline-block;vertical-align:middle;width:90px;height:38px;position:relative;overflow:hidden;cursor:pointer;border:1px solid #CCC;-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.chSwSlLe,.chSwSlRi{position:absolute;top:0;cursor:pointer;font-family:sans-serif;font-weight:700;text-align:center;color:#FFF}.chSwSlRi{background:#a5a5a5}.chSwSlLe{background:#3385d2}.chSwSl{cursor:pointer;position:absolute;}.chSwSlH{position:absolute;top:0;left:0;background-color:#CCC;height:0;}</style>').insertBefore('head link[rel=stylesheet], head style');
			}

			// Insert into DOM
			$elm.insertAfter($this)
			
			if($this.attr('id')) {
				$elm.attr('id', $this.attr('id')+'-chSw');
			}
			
			var h = $elm.height(),
				w = $elm.width();
			
			// Element sizes (dynamic)
			$txts.css({
				fontSize: (h * 0.4)+'px',
				lineHeight: h+'px',
				height: h,
				width: w
			});
			
			$txtOff.css({ 'left': (w * 0.2) });
			$txtOn.css({ 'left': (w * 0.2) - w });
			
			$slider.css({
				width: w * 0.4,
				height: h
			});
			if(!$handle.height()) {
				$handle.css('height', $elm.height());
			}
			if(!$handle.width()) {
				$handle.css('width', (w * 0.4));
			}
			
			max = $elm.width() - $slider.outerWidth();

			// Center text display
			$('.chSwSlLe,.chSwSlRi',$elm)

			var beginDragFn = this.beginDrag, dragFn = this.drag, endDragFn = this.endDrag;
			// Bind events-- detect touch browser
			if(('ontouchstart' in window) || window.DocumentTouch && document instanceof DocumentTouch) {
				fixTimeoutIntervalBug();
				
				// Dragging started
				$slider.on('touchstart', function(e){
					beginDragFn(e.originalEvent.touches[0].pageX);
				});
				
				// Dragging slider
				$(window).on('touchmove', function(e){
					dragFn(e.originalEvent.touches[0].pageX);
				}).on('touchend touchcancel', function(e){
					// End drag
					e.preventDefault();
					endDragFn();
				});
			} else {
				// Turn off/on when clicked
				$txtOn.on('click', this.turnOn);
				$txtOff.on('click', this.turnOff);
				$slider.on('click', this.toggle);
				
				// Beginning drag
				$slider.on('mousedown', function(e){
					beginDragFn(e.pageX);
				});
				// Dragging slider
				$(window).on('mousemove', function(e){
					dragFn(e.pageX);
				}).on('mouseup', function(e){
					e.preventDefault();
					endDragFn();
				}).on('mouseleave', function(e) {
					if (!e.relatedTarget || e.toElement.nodeName == "HTML") {
						e.preventDefault();
						endDragFn();
					}
				});
			}
			
			// Change the switch if we change the <input>
			$this.on('change', this.update);

			// Set initial state (checked or not)
			if($this.prop('checked')) { $slider.css({ 'left': max }); }
			
			// Trigger when 'checked' property changed
			$.propHooks.checked = $.attrHooks.checked = {
			    set: function (el, value) {
			        el.checked = value;
					if(!ignoreUpdate) {
						$(el).trigger('change');
					} else {
						ignoreUpdate = false;
					}
			    }
			}
		});
	}
	
	var fixTimeoutIntervalBug = function() {
		// This library re-implements setTimeout, setInterval, clearTimeout, clearInterval for iOS6.
		// iOS6 suffers from a bug that kills timers that are created while a page is scrolling.
		// This library fixes that problem by recreating timers after scrolling finishes (with interval correction).
		// This code is free to use by anyone (MIT, blabla).
		// Author: rkorving@wizcorp.jp
		(function (window) {

			var timeouts = {};
			var intervals = {};
			var orgSetTimeout = window.setTimeout;
			var orgSetInterval = window.setInterval;
			var orgClearTimeout = window.clearTimeout;
			var orgClearInterval = window.clearInterval;


			function createTimer(set, map, args) {
				var id, cb = args[0], repeat = (set === orgSetInterval);

				function callback() {
					if(cb) {
						cb.apply(window, arguments);
						if(!repeat) {
							delete map[id];
							cb = null;
						}
					}
				}

				args[0] = callback;

				id = set.apply(window, args);

				map[id] = { args:args, created:Date.now(), cb:cb, id:id };

				return id;
			}


			function resetTimer(set, clear, map, virtualId, correctInterval) {
				var timer = map[virtualId];

				if (!timer) {
					return;
				}

				var repeat = (set === orgSetInterval);

				// cleanup

				clear(timer.id);

				// reduce the interval (arg 1 in the args array)

				if (!repeat) {
					var interval = timer.args[1];

					var reduction = Date.now() - timer.created;
					if (reduction < 0) {
						reduction = 0;
					}

					interval -= reduction;
					if (interval < 0) {
						interval = 0;
					}

					timer.args[1] = interval;
				}

				// recreate

				function callback() {
					if (timer.cb) {
						timer.cb.apply(window, arguments);
						if (!repeat) {
							delete map[virtualId];
							timer.cb = null;
						}
					}
				}

				timer.args[0] = callback;
				timer.created = Date.now();
				timer.id = set.apply(window, timer.args);
			}


			window.setTimeout = function () {
				return createTimer(orgSetTimeout, timeouts, arguments);
			};


			window.setInterval = function () {
				return createTimer(orgSetInterval, intervals, arguments);
			};

			window.clearTimeout = function (id) {
				var timer = timeouts[id];

				if (timer) {
					delete timeouts[id];
					orgClearTimeout(timer.id);
				}
			};

			window.clearInterval = function (id) {
				var timer = intervals[id];

				if (timer) {
					delete intervals[id];
					orgClearInterval(timer.id);
				}
			};

			window.addEventListener('scroll', function () {
				// recreate the timers using adjusted intervals
				// we cannot know how long the scroll-freeze lasted, so we cannot take that into account

				var virtualId;

				for (virtualId in timeouts) {
					resetTimer(orgSetTimeout, orgClearTimeout, timeouts, virtualId);
				}

				for (virtualId in intervals) {
					resetTimer(orgSetInterval, orgClearInterval, intervals, virtualId);
				}
			});

		}(window));
	}
})(jQuery);

